#include "eudoraToBeMail.h"

void eudoraBoxToBeMail (char *mailBoxFileNames)
{	
	FILE	*tocFile,	//File pointer to a TOC file
			*mbxFile;	//File pointer to an MBX file
	char	tocFileName[B_FILE_NAME_LENGTH];
	char	mbxFileName[B_FILE_NAME_LENGTH];
	struct	TOCHeaderBlock mailBoxHeader;	//For storing the header block info for a Eudora mailbox
	struct	TOCMessageBlock messageHeader;	//For storing the message header information for a Eudora message
	char	fileBuffer[FILE_READ_BUFFER_SIZE];	//For buffering file read/write data.
	unsigned long int	amountRead;		//stores the amount read from an mbx file entry.
	
	char	newFileName[B_FILE_NAME_LENGTH];	//The name of the new mail file.
	char	newFileEntryName[2*B_FILE_NAME_LENGTH + 3];
	BDirectory	newDirectory;
	BEntry	fileSystemEntry;	//The location of the new file.
	BFile	newFile;			//The new file being created.
	BNode	fileSystemNode;		//Datapoint in the file System.
	BNodeInfo	fileInfo;		//Information about the node.
	char	tmpStr[MAX_FROM_FIELD_SIZE];			//Temporary character pointer
	bool	foundFrom, foundSubject, foundTo, foundCc, foundReply_to;

	//Create toc filename:
	strcpy (tocFileName, mailBoxFileNames);
	strcpy ((tocFileName + strlen (tocFileName)), ".toc");
	//Create mbx filename:
	strcpy (mbxFileName, mailBoxFileNames);
	strcpy ((mbxFileName + strlen (mbxFileName)), ".mbx");
	
	//Open the eudora files:
	tocFile = fopen (tocFileName, "rb");
	mbxFile = fopen (mbxFileName, "rb");
	
	//First we need to find out how many messages exist in this mailbox:
	getTOCHeaderBlock (tocFile, mailBoxHeader);
	
	printf ("Converting mailbox: %s...\n", mailBoxHeader.mailBoxName);

	//Create subdirectory for mailbox:
	if (newDirectory.CreateDirectory (mailBoxHeader.mailBoxName, NULL) != B_OK)
		printf ("BDirectory CreateDirectory (...) returned an error...\n");

	for (unsigned int msgNum = 1; msgNum <= mailBoxHeader.numMessages; msgNum++)
	{	//Get Eudora msg block information for this message:
		getTOCMessageBlock (tocFile, msgNum, messageHeader);

		//Create the new fileName: <MailBoxName>:<MsgNum> - <From>
		//Set the BEntry object to the location of the new file:
		strcpy (newFileName, mailBoxHeader.mailBoxName);	//<MailBoxName>
		strcpy (newFileName + strlen (newFileName), ":");	//:
		sprintf (newFileName + strlen (newFileName), "%d", msgNum);//<MsgNum>	
		strcpy (newFileName + strlen (newFileName), " - ");	// - 
		strcpy (newFileName + strlen (newFileName), messageHeader.sender);	//<From>

		//Create an entry object to describe the new mail file:
		strcpy (newFileEntryName, "./");
		strcpy ((newFileEntryName + strlen (newFileEntryName)), mailBoxHeader.mailBoxName);
		strcpy ((newFileEntryName + strlen (newFileEntryName)), "/");
		strcpy ((newFileEntryName + strlen (newFileEntryName)), newFileName);
		
		if (fileSystemEntry.SetTo (newFileEntryName, false) != B_OK)
			printf ("BEntry SetTo(...) returned an error...\n");
		//Create the new file, physically:
		if (newFile.SetTo(&fileSystemEntry, B_WRITE_ONLY | B_CREATE_FILE | B_FAIL_IF_EXISTS) != B_OK)
			printf ("BFile SetTo() returned an error...\n");
		//Prepare for working with file info & attr's:
		if (fileSystemNode.SetTo (&fileSystemEntry) != B_OK)
			printf ("BNode SetTo(...) returned an error...\n");
		//Write the filetype attribute:	
		/*if (*/fileSystemNode.WriteAttr ("BEOS:TYPE", B_STRING_TYPE, 0, "text/x-email", strlen ("text/x-e-mail") + 1);/* != B_OK)*/
		//	printf ("BNode WritAttr(...) returned an error for BEOS:TYPE...\n");

		//Write data to the file:
		amountRead = 0;
		strcpy (fileBuffer, "initValue");
		foundFrom = foundSubject = foundTo = foundCc = foundReply_to = false;
		while (amountRead < messageHeader.mbxMsgLength && strlen (fileBuffer) != 0)
		{	//Seek to and read one buffer's worth of data:
			fseek (mbxFile, messageHeader.mbxOffset + amountRead, SEEK_SET);
			fgets (fileBuffer, sizeof (fileBuffer), mbxFile);
			
			amountRead += strlen (fileBuffer);

			//From:
			if (memcmp (fileBuffer, From, sizeof (From)) == 0 && !foundFrom)
			{	
				foundFrom = true;
				//Fill in the MAIL:from attribute:
				strcpy (tmpStr, fileBuffer + sizeof (From) + 1);
				tmpStr[strlen (tmpStr) - 2] = '\0'; 
				if (fileSystemNode.WriteAttr("MAIL:from", B_STRING_TYPE, 0, tmpStr, strlen (tmpStr) + 1) != strlen (tmpStr) + 1)
					printf ("BNode WriteAttr(...) returned an error for MAIL:from...\n");
			}//End if From
			//Subject:
			else if (memcmp (fileBuffer, Subject, sizeof (Subject)) == 0 && !foundSubject)
			{
				foundSubject = true;
				//Fill in the MAIL:subject attribute:
				strcpy (tmpStr, fileBuffer + sizeof (Subject) + 1);
				tmpStr[strlen (tmpStr) - 2] = '\0';
				//Subject:
				if (fileSystemNode.WriteAttr("MAIL:subject", B_STRING_TYPE, 0, tmpStr, strlen (tmpStr) + 1) != strlen (tmpStr) + 1)
					printf ("BNode WriteAttr(...) returned an error for MAIL:subject...\n");
			}//End if Subject
			//To:
			else if (memcmp (fileBuffer, To, sizeof (To)) == 0 && !foundTo)
			{
				foundTo = true;
				//Fill in the MAIL:to attribute:
				strcpy (tmpStr, fileBuffer + sizeof (To) + 1);
				tmpStr[strlen (tmpStr) - 2] = '\0';
				if (fileSystemNode.WriteAttr("MAIL:to", B_STRING_TYPE, 0, tmpStr, strlen (tmpStr) + 1) != strlen (tmpStr) + 1)
					printf ("BNode WriteAttr(...) returned an error for MAIL:to...\n");
			}//End if To
			//Cc:
			else if (memcmp (fileBuffer, Cc, sizeof (Cc)) == 0 && !foundCc)
			{
				foundCc = true;
				//Fill in the MAIL:cc attribute:
				strcpy (tmpStr, fileBuffer + sizeof (Cc) + 1);
				tmpStr[strlen (tmpStr) - 2] = '\0';
				if (fileSystemNode.WriteAttr("MAIL:cc", B_STRING_TYPE, 0, tmpStr, strlen (tmpStr) + 1) !=strlen (tmpStr) + 1)
					printf ("BNode WriteAttr(...) returned an error for Mail:cc...\n");
			}//End if Cc
			else if (memcmp (fileBuffer, Reply_to, sizeof (Reply_to)) == 0 && !foundReply_to)
			{
				foundReply_to = true;
				//Fill in the MAIL: attribute:
				strcpy (tmpStr, fileBuffer + sizeof (Reply_to) + 1);
				tmpStr[strlen (tmpStr) - 2] = '\0';	
				if (fileSystemNode.WriteAttr("MAIL:reply", B_STRING_TYPE, 0, tmpStr, strlen (tmpStr) + 1) !=strlen (tmpStr) + 1)
					printf ("BNode WriteAttr(...) returned an error for Mail:reply...\n");
			}//End if Reply-to
			
			//Write the last-read line to the new BeMail file:
			if (newFile.Write (fileBuffer, strlen (fileBuffer)) != strlen (fileBuffer))
				printf ("BFile Write () could not write all bytes...\n");
		}//end while for reading and writing e-mail message text.

		//Fill in the MAIL:name attribute:
		//We use the Eudora Header for this attribute for convenience:
		if (fileSystemNode.WriteAttr("MAIL:name", B_STRING_TYPE, 0, messageHeader.sender, strlen (messageHeader.sender) + 1) != strlen (messageHeader.sender) + 1)
			printf ("BNode WritAttr(...) returned an error for MAIL:name...\n");		
		//Status
		if (messageHeader.msgStatus == 0)
			strcpy (tmpStr, "New");
		else	strcpy (tmpStr, "Read");
		if (fileSystemNode.WriteAttr("MAIL:status", B_STRING_TYPE, 0, tmpStr, strlen (tmpStr) + 1) != strlen (tmpStr)+1)
			printf ("BNode WriteAttr(...) returned an error for MAIL:status...\n");
		//Priority
		//if (fileSystemNode.WriteAttr("MAIL:priority", B_STRING_TYPE, 0, "thePriorityAttr", strlen ("thePriorityAttr")+1) !=strlen ("thePriorityAttr")+1)
		//	printf ("BNode WriteAttr(...) returned an error for Priority...\n");
		//When
		/*if (*/fileSystemNode.WriteAttr ("MAIL:when", B_TIME_TYPE, 0, &messageHeader.msgDate, sizeof (messageHeader.msgDate)); /*!= B_OK)*/
		//	printf ("BNode WriteAttr(...) returned an error for MAIL:when...\n");
	}//End For loop (each message)
	
	fclose (tocFile);
	fclose (mbxFile);
}